import org.junit.Assert;
import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.KieServices;
import org.kie.api.builder.KieBuilder;
import org.kie.api.builder.KieFileSystem;
import org.kie.api.builder.KieModule;
import org.kie.api.builder.KieRepository;
import org.kie.api.builder.Message;
import org.kie.api.builder.ReleaseId;
import org.kie.api.definition.type.FactType;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;

public class TestRule {
	
	private static final String RULE_NOTWORKING = "package redhat\n" +
            "declare Fact\n" +
            "    integer : int\n" +
            "    string1 : String\n" +
            "    string2 : String\n" +
            "end\n" +
            "rule \"Rule\"\n" +
            "when\n" +
            "forall (Fact(string2 == \"Y\"))\n" +
            "(\n" +
            "    forall (Fact(integer == 42)) or\n" +
            "    Fact(integer == 43)\n" +
            ")\n" +
            "forall (Fact(string1 == \"X\"))\n" +
            "then\n" +
            "end";       
    
    private static final String RULE_WORKING = "package redhat\n" +
            "declare Fact\n" +
            "    integer : int\n" +
            "    string1 : String\n" +
            "    string2 : String\n" +
            "end\n" +
            "rule \"Rule\"\n" +
            "when\n" +
            "forall (Fact(string1 == \"X\"))\n" +
            "forall (Fact(string2 == \"Y\"))\n" +
            "(\n" +
            "    forall(Fact(integer == 42)) or\n" +
            "    Fact(integer == 43)\n" +
            ")\n" +
            "then\n" +
            "end";   
    
	@Test
	public void testRuleWorking() throws Exception {
	    doTest(RULE_WORKING);
	}
	
	@Test
	public void testRuleNotWorking() throws Exception {
	    doTest(RULE_NOTWORKING);
	}
	
	public void doTest(String strRule) throws InstantiationException, IllegalAccessException{
		
	    KieServices kieServices = KieServices.Factory.get();
	    KieRepository kieRepository = kieServices.getRepository();
	    KieModule kieModule = buildKieModule("1.0.0", strRule);
	    KieContainer kieContainer = kieServices.newKieContainer(kieModule.getReleaseId());
	    kieRepository.removeKieModule(kieModule.getReleaseId());
	    KieBase kieBase = kieContainer.getKieBase();
	    KieSession kieSession = kieBase.newKieSession();

        FactType factType = kieBase.getFactType("redhat", "Fact");
        
        Object fact = factType.newInstance();
        factType.set(fact, "string1", "X");
        factType.set(fact, "string2", "Y");
        factType.set(fact, "integer", 42);

        kieSession.insert(fact);

        int n = kieSession.fireAllRules();
        Assert.assertEquals(1, n);
	}
	
	private static KieModule buildKieModule(String version, String rules) {
	    KieServices ks = KieServices.Factory.get();
	    KieFileSystem kfs = ks.newKieFileSystem();
	
	    ReleaseId rid = ks.newReleaseId("org.redhat", "sample", version);
	    kfs.generateAndWritePomXML(rid);
	
	    kfs.write("src/main/resources/rules.drl", rules);
	
	    KieBuilder kb = ks.newKieBuilder(kfs);
	    kb.buildAll(); 
	    if (kb.getResults().hasMessages(Message.Level.ERROR)) {
	        throw new RuntimeException("Build Errors:\n" + kb.getResults().toString());
	    }
	    return kb.getKieModule();
	}
}
